get_MRfulltable <- 
  function(obj, by = 2, coef = NULL, number = 10, taxa = obj@taxa, 
           uniqueNames = FALSE, adjustMethod = "fdr", group = 0, eff = 0, 
           numberEff = FALSE, ncounts = 0, file = NULL){
    if (length(grep("fitFeatureModel", obj@call))) {
      groups = factor(obj@design[, by])
      by = "logFC"
      coef = 1:2
      tb = data.frame(logFC = obj@fitZeroLogNormal$logFC, se = obj@fitZeroLogNormal$se)
      p = obj@pvalues
    }
    else {
      tb = obj@fit$coefficients
      if (is.null(coef)) {
        coef = 1:ncol(tb)
      }
      p = obj@eb$p.value[, by]
      groups = factor(obj@fit$design[, by])
      if (eff > 0) {
        effectiveSamples = calculateEffectiveSamples(obj)
        if (numberEff == FALSE) {
          valid = which(effectiveSamples >= quantile(effectiveSamples, 
                                                     p = eff, na.rm = TRUE))
        }
        else {
          valid = which(effectiveSamples >= eff)
        }
      }
    }
    tx = as.character(taxa)
    if (uniqueNames == TRUE) {
      for (nm in unique(tx)) {
        ii = which(tx == nm)
        tx[ii] = paste(tx[ii], seq_along(ii), sep = ":")
      }
    }
    padj = p.adjust(p, method = adjustMethod)
    cnts = obj@counts
    yy = cnts > 0
    pa = matrix(unlist(fitPA(obj@counts, groups)), ncol = 5)
    np0 = rowSums(yy[, groups == 0])
    np1 = rowSums(yy[, groups == 1])
    nc0 = rowSums(cnts[, groups == 0])
    nc1 = rowSums(cnts[, groups == 1])
    if (group == 0) {
      srt = order(abs(tb[, by]), decreasing = TRUE)
    }
    else if (group == 1) {
      srt = order((tb[, by]), decreasing = TRUE)
    }
    else if (group == 2) {
      srt = order((tb[, by]), decreasing = FALSE)
    }
    else if (group == 3) {
      srt = order(p, decreasing = FALSE)
    }
    else {
      srt = 1:length(padj)
    }
    valid = 1:length(padj)
    if (ncounts > 0) {
      np = rowSums(cbind(np0, np1))
      valid = intersect(valid, which(np >= ncounts))
    }
    srt = srt[which(srt %in% valid)][1:min(number, nrow(tb))]
    mat = cbind(np0, np1)
    mat = cbind(mat, nc0)
    mat = cbind(mat, nc1)
    mat = cbind(mat, pa)
    mat = cbind(mat, tb[, coef])
    mat = cbind(mat, p)
    mat = cbind(mat, padj)
    rownames(mat) = tx
    mat = mat[srt, ]
    nm = c("+samples in group 0", "+samples in group 1", "counts in group 0", 
           "counts in group 1", c("oddsRatio", "lower", "upper", 
                                  "fisherP", "fisherAdjP"), colnames(tb)[coef], "pvalues", 
           "adjPvalues")
    colnames(mat) = nm
    if (!is.null(file)) {
      nm = c("Taxa", nm)
      mat2 = cbind(rownames(mat), mat)
      mat2 = rbind(nm, mat2)
      write(t(mat2), ncolumns = ncol(mat2), file = file, sep = "\t")
    }
    return(as.data.frame(mat))
  }
# Function to perform relevant test (`fitZig`)
# - `MGS` - A metagenomeSeq object, most-likely produced using the conversion tool make_metagenomeSeq()
# - `variable` - the variable among the sample_data (aka "phenoData" in metagenomeSeq) that you want to test
# - `physeq` - optional phyloseq data object that has the relevant `tax_table`.
# `phyloseq` or `taxonomyTable` is fine.
test_metagenomeSeq = function(MGS, variable, physeq=NULL){
  require("metagenomeSeq")
  require("phyloseq")
  # Create the `mod` variable used in the fitZig test.
  if( inherits(variable, "factor") ){
    # If variable is already a factor, use directly in model.matrix
    mod = model.matrix( ~variable )
  } else if( inherits(variable, "matrix") ){
    # If it is a matrix, assume that model.matrix() has been used already
  } else if( inherits(variable, "character") ){ 
    # If it is a character that specifies a variable in phenoData, 
    # use the corresponding variable from MGS
    if( variable %in% colnames(phenoData(MGS)@data) ){
      mod = model.matrix(~phenoData(MGS)@data[, variable])
    } else {
      stop("The phenoData variable name you specified is not present in `phenoData(MGS)`")
    }
  } else {
    stop("Improper specification of the experimental design variable for testing. See `variable` argument")
  }
  
  # Wrapper to run the Expectation-maximization algorithm 
  # and estimate $f_count$ fits with the zero-inflated Guassian (z.i.g.)
  # fit = fitZig(MGS, mod)
  fit = fitFeatureModel(MGS, mod)
  
  # You need to specify all OTUs to get the full table from MRfulltable. 
  x = get_MRfulltable(obj = fit, number=nrow(assayData(MGS)$counts))
  # x = MRfulltable(obj = fit, number=nrow(assayData(MGS)$counts))
  
  # if any OTUs left out, rm those from x. Detected by NA rownames.
  x = x[!is.na(rownames(x)), ]
  # Modify this data.frame by adding the OTUnames. Clip the ":1" added to the OTU names
  rownames(x) <- gsub(":1", "", x=rownames(x), fixed=TRUE)
  x$OTUnames <- as.character(rownames(x))
  if( !is.null(tax_table(physeq, errorIfNULL=FALSE)) ){
    # Attach the bacterial taxonomy to the table, if available
    TAX = data.frame(tax_table(physeq))
    TAX$OTUnames <- as.character(rownames(TAX))    
    y = merge(x, TAX, by="OTUnames") 
  } else {
    y = x
  }
  # Sort and return
  y = y[order(y$adjPvalue), ]
  return(y)
}
